/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 Muli Ben-Yehuda, mulix@actcom.co.il      */
/* License: GNU General Public License                              */

/*
 * $Id: special_hijack.c,v 1.6 2002/07/01 00:10:21 mulix Exp $
 */

/* 
 * This file has the hijacking functions for special system calls, such 
 * as exit(), execve() or fork(). 
 */ 

#include "linux/sched.h"

/* exit, syscall 1 */ 
static asmlinkage long stub_exit(int error_code)
{
	typedef int (*stub_exit)(int error_code); 
	typedef int (*stub_before_exit)(int error_code); 
	stub_exit orig_syscall_func;
	stub_before_exit before_func;
	struct syscall_hijack_data* hdata = &hijack_data[1];

	down(&hdata->sem);
	hdata->invocation_ref_count++;
	up(&hdata->sem);
	before_func = (stub_before_exit)hdata->before_hijack_func;

	if (before_func)
		(*before_func)(error_code);

	orig_syscall_func = (stub_exit)hdata->orig_syscall;

	down(&hdata->sem);
	if (--hdata->invocation_ref_count == 0 && hdata->defered_release_flag == 1) {
	    /* we couldn't reduce the module's use count when the syscall  */
	    /* was originally released (as marked by the 'defered' flag) - */
	    /* so we do so now.                                            */
	    MOD_DEC_USE_COUNT;
	    hdata->defered_release_flag = 0;
	}
	up(&hdata->sem);
	/* WE HAVE A RACE FROM HERE ON - we're still executing on behalf of the stub_exit
	 * code, but no longer protected by the invocation_ref_count. Since the original 
	 * syscall never returns, I don't see a way to avoid this race (except for disallowing  
	 * unloading the module, which we already do)
	 */

	 /* finally, call the syscall itself */ 
	if (orig_syscall_func)
		(*orig_syscall_func)(error_code); 

	/* exit() must not return! */ 
	BUG(); 
	
	/* doesn't mean anything, silence gcc */ 
	return 0; 
}

void init_hijack_syscalls(void)
{
	stub_functions[1] = (func_ptr)stub_exit; 
 	stub_functions[2] = (func_ptr)stub_syscall_2;
 	stub_functions[11] = (func_ptr)stub_syscall_11;
}
