/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 Muli Ben-Yehuda, mulix@actcom.co.il      */
/* License: GNU General Public License                              */

/* 
 * $Id: tests.c,v 1.7 2002/08/23 10:20:33 mulix Exp $
 */ 

#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <stdio.h>

#include "tests.h" 

struct test tests[] = {
	{ "all_syscalls_register_rules", SCT_SUBSYS_MODULE, all_syscalls_register_rules }, 
	{ "simple_syscall_usage", SCT_SUBSYS_MODULE, test_simple_syscall_usage }, 
	{ "rule_register_deregister", SCT_SUBSYS_MODULE, test_rule_register_deregister }, 
	{ "all_syscalls_usage", SCT_SUBSYS_MODULE, test_all_syscalls_usage }, 
	
	{ "filter_eval_bit", SCT_SUBSYS_RULES, test_filter_eval_bit }, 
	{ "eval_complex_filter", SCT_SUBSYS_RULES, test_eval_complex_filter }, 
	{ "filter_serialize_deserialize", SCT_SUBSYS_RULES, test_filter_serialize_deserialize },
	{ "rules_serialize_deserialize", SCT_SUBSYS_RULES, test_rules_serialize_deserialize },
	{ "tracking_rule_add_delete", SCT_SUBSYS_RULES, test_tracking_rule_add_delete },
	{ "tracking_rule_add_delete_empty_list", SCT_SUBSYS_RULES, test_tracking_rule_add_delete_empty_list },
	{ "indexed_field_variables", SCT_SUBSYS_RULES, test_indexed_field_variables },
	{ "kernel_variables", SCT_SUBSYS_RULES, test_kernel_variables },
	{ "short_circuiting", SCT_SUBSYS_RULES, test_short_circuiting },
	{ "filter_node_vectors", SCT_SUBSYS_RULES, test_filter_node_vectors },
	{ "membuf", SCT_SUBSYS_RULES, test_membuf }, 
	{ "tracking_rule_list", SCT_SUBSYS_RULES, test_tracking_rule_list }, 
	{ "logging", SCT_SUBSYS_RULES, test_logging }, 
	{ "get_rules", SCT_SUBSYS_RULES | SCT_SUBSYS_MODULE, test_get_rules }, 
	{ "void_ptr_buffer", SCT_SUBSYS_RULES, test_void_ptr_buffer }, 
	{ "buffer_match", SCT_SUBSYS_RULES, test_buffer_match }, 
	{ "membuf_enlarge", SCT_SUBSYS_RULES, test_membuf_enlarge }, 
	{ "ring_membuf_exact", SCT_SUBSYS_RULES, test_ring_membuf_exact }, 
	{ "ring_membuf_drop_new", SCT_SUBSYS_RULES, test_ring_membuf_drop_new }, 
	{ "ring_membuf_drop_old", SCT_SUBSYS_RULES, test_ring_membuf_drop_old }, 
	{ "cyclic_membuf", SCT_SUBSYS_RULES, test_cyclic_membuf }, 
	{ "logging_cyclic", SCT_SUBSYS_RULES, test_logging_cyclic }, 
	{ "struct", SCT_SUBSYS_RULES, test_struct }, 
	{ "long_long", SCT_SUBSYS_RULES, test_long_long}, 
	{ NULL, 0, NULL } 
}; 

struct test* find_test(const char* name)
{
	struct test* p; 
	
	for (p = &tests[0]; p && p->test_fn; ++p) {
		if (!strcmp(p->name, name))
			return p; 
	}
	return NULL; 

}

const char* subsystem_as_str(unsigned int subsys)
{
	/* if we add more subsstems, rewrite me */ 

	if (subsys == (SCT_SUBSYS_RULES | SCT_SUBSYS_MODULE))
		return "module, rules"; 
	
	if (subsys == SCT_SUBSYS_RULES)
		return "rules"; 

	if (subsys == SCT_SUBSYS_MODULE)
		return "module"; 

	return "unknown"; 
}

unsigned int str_to_subsystem(const char* str)
{
	assert(str); 
	
	if (!strcmp(str, "module"))
		return SCT_SUBSYS_MODULE; 

	if (!strcmp(str, "rules"))
		return SCT_SUBSYS_RULES; 

	return 0; 
}

void list_tests(FILE* f)
{
	struct test* p; 
	
	for (p = &tests[0]; p && p->test_fn; ++p)
		fprintf(f, "%s (%s)\n", p->name, 
			subsystem_as_str(p->subsystem));
}
