/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 Muli Ben-Yehuda, mulix@actcom.co.il	    */
/* License: GNU General Public License                              */

/*
 * $Id: sct_logctrl.c,v 1.2 2002/07/10 15:26:17 mulix Exp $
 */

#include <unistd.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>

#include "sct_rules_module/module_interface.h"

void usage(const char* prog)
{
	printf("Usage: %s cmd\n", prog); 
	exit (EXIT_FAILURE); 
}

int main(int argc, char** argv)
{
	char* scmd; 
	int cmd; 
	int fd; 

	if (argc < 2) 
		usage(argv[0]); 

	scmd = argv[1]; 

	if ((fd = open("/dev/sct_log", 0)) < 0) {
		perror("open /dev/sct_log"); 
		exit (EXIT_FAILURE); 
	}
	
	if (!strcmp(scmd, "drop_old")) 
		cmd = CTL_IOC_DROP_OLD;
	else if (!strcmp(scmd, "drop_new")) 
		cmd = CTL_IOC_DROP_NEW; 
	else if (!strcmp(scmd, "flush"))
		cmd = CTL_IOC_FLUSH; 
	else if (!strcmp(scmd, "enable"))
		cmd = CTL_IOC_ENABLE_DEV; 
	else if (!strcmp(scmd, "disable"))
		cmd = CTL_IOC_DISABLE_DEV; 
	else
		usage(argv[0]); 

	printf("command is %s (%d)\n", scmd, cmd); 
	ioctl(fd, cmd); 
	perror("ioctl"); 

	return EXIT_SUCCESS; 
}
